@@ -1,4 +1,4 @@
-/* --< GKrellShoot 0.4.1 >--{ 17 August 2002 }--
+/* --< GKrellShoot 0.4.3 >--{ 19 November 2004 }--
  * Ported to Gkrellm2.0
  *
  * Author: M.R.Muthu Kumar (m_muthukumar@users.sourceforge.net)
@@ -19,15 +19,15 @@
 
 #include<math.h>
 
-#define GKRELLSHOOT_VER 	"0.4.1"
+#define GKRELLSHOOT_VER 	"0.4.3"
 #define DEFAULT_XLOCK		"xscreensaver-command -lock"
 #define SHOOT_SCREEN		"import -window root"
 #define SHOOT_WINDOW		"import"
-#define DEFAULT_VIEW		"ee"
+#define DEFAULT_VIEW		"display"
 #define DEFAULT_IMAGE		"jpg"
 #define WINDOW_LABEL	"Grab selected window/area ( unchecking this will grab whole screen)"
 
-#define DEFAULT_OUTFILE	"/tmp/mk.jpg"
+#define DEFAULT_OUTFILE	"mk.jpg"
 #define	CONFIG_NAME	"GkrellShoot"	/* Name in the configuration window */
 #define	STYLE_NAME	"GkrellShoot"	/* Theme subdirectory name */
 			                /*  and gkrellmrc style name.*/ 
@@ -54,6 +54,9 @@
 #define MIN_PANELS	0
 #define NUM_COLORS 32
 
+#define SELECT_BOTH	0
+#define SELECT_LOCK	1
+#define SELECT_SHOOT	2
 
 static gchar *anim_name[] = { B_BALL, MESH, RADAR, SINE, STAR, RAIN, R_LINE,
                               C_BOARD, SCANNER, C_BAR, R_STAR };
@@ -64,10 +67,10 @@ static GkrellmMonitor		*mon;
 static GkrellmChart		*chart[MAX_PANELS];
 static GkrellmChartconfig	*chart_config=NULL;
 
-static GkrellmPanel	*panel;
+static GkrellmPanel	*panel[3];
 
-static GkrellmDecal	*decal_lock,
-		*decal_shoot;
+static GkrellmDecal	*decal_lock[2],
+			*decal_shoot[2];
 
 static gint		chart_w = 60;
 static gint		style_id;
@@ -76,10 +79,13 @@ static gint wait_seconds;
 static gint view_image;
 static gint window_or_full;
 static gint with_frame;
+static gint grayscale;
 static gchar xlock_cmd[513];
 static gchar view_cmd[513];
+static gchar save_dir[513];
 static gchar image_format[32];
 static gchar shoot_cmd[1024];
+static gint  lock_shoot_select = SELECT_BOTH;
 
 static gint active_panels=1;
 static gint sel_num_panels=1;
@@ -99,7 +105,10 @@ static GtkWidget	*wait_seconds_option,
                         *cycle_anim_option[MAX_PANELS],
                         *view_image_option,
                         *xlock_cmd_option,
-                        *frame_option;
+                        *save_dir_option,
+                        *frame_option,
+                        *grayscale_option,
+			*lock_shoot_option[3];
 
 static GtkWidget *laptop = NULL;
 
@@ -141,7 +150,7 @@ static gint r_g_b[][3] = {
                          	{ 46 , 139, 87  }
                          };
 
-gchar filename[512];
+gchar filename[1024];
 int load_val = 50;
 struct tm  *tm;
 
@@ -1225,13 +1234,17 @@ chart_expose_event(GtkWidget *widget, GdkEventExpose *ev, gpointer echart )
 }
 
 static gint
-expose_event (GtkWidget *widget, GdkEventExpose *ev)
+expose_event (GtkWidget *widget, GdkEventExpose *ev, gpointer epanel)
 {
-    if (widget == panel->drawing_area)
+  gint i;
+
+  i = GPOINTER_TO_INT( epanel );
+
+    if (widget == panel[i]->drawing_area)
     {
         gdk_draw_pixmap(widget->window,
             widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-            panel->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
+            panel[i]->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
             ev->area.width, ev->area.height);
     }
     return FALSE;
@@ -1343,6 +1356,7 @@ make_shoot_cmd(void)
 
    gchar tmp_scmd[32];
    gchar tmp_fcmd[32];
+   gchar tmp_gcmd[32];
 
    if ( wait_seconds > 0 )
    {
@@ -1353,13 +1367,19 @@ make_shoot_cmd(void)
    if ( with_frame ) { sprintf( tmp_fcmd, " %s ", "-frame" ); }
    else { sprintf( tmp_fcmd, "%s", " " ); }
 
+   if ( grayscale ) 
+   { 
+     sprintf( tmp_gcmd, " %s ", "-colorspace GRAY -depth 8" ); 
+   }
+   else { sprintf( tmp_gcmd, "%s", " " ); }
+
    if ( window_or_full ) 
    {
-     sprintf( tmp_wcmd,"%s %s %s ",  SHOOT_WINDOW, tmp_fcmd, filename );
+     sprintf( tmp_wcmd,"%s %s %s %s ",  SHOOT_WINDOW, tmp_fcmd, tmp_gcmd, filename );
    }
    else 
    {
-     sprintf( tmp_wcmd,"%s %s %s ",  SHOOT_SCREEN, tmp_fcmd, filename );
+     sprintf( tmp_wcmd,"%s %s %s %s ",  SHOOT_SCREEN, tmp_fcmd, tmp_gcmd, filename );
    }
 
    if ( view_image )
@@ -1387,7 +1407,7 @@ run_shoot_cmd()
   tm = gkrellm_get_current_time();
 
   sprintf( filename, "%s/gkrellShoot_%02d-%02d-%02d_%02d%02d%02d.%s", 
-		  gkrellm_homedir(),
+		  save_dir,
 		  tm->tm_mon+1, tm->tm_mday, tm->tm_year-100, tm->tm_hour, 
 		  tm->tm_min, tm->tm_sec, image_format );
 
@@ -1414,6 +1434,30 @@ cb_button(GkrellmDecalbutton *button)
   }
 }
 
+static void
+show_lock_shoot_select()
+{
+
+   if ( lock_shoot_select == SELECT_BOTH )
+   {
+      gkrellm_panel_show( panel[SELECT_BOTH] );
+      gkrellm_panel_hide( panel[SELECT_LOCK] );
+      gkrellm_panel_hide( panel[SELECT_SHOOT] );
+
+   }
+   if ( lock_shoot_select == SELECT_LOCK )
+   {
+      gkrellm_panel_show( panel[SELECT_LOCK] );
+      gkrellm_panel_hide( panel[SELECT_BOTH] );
+      gkrellm_panel_hide( panel[SELECT_SHOOT] );
+   }
+   if ( lock_shoot_select == SELECT_SHOOT )
+   {
+      gkrellm_panel_show( panel[SELECT_SHOOT] );
+      gkrellm_panel_hide( panel[SELECT_BOTH] );
+      gkrellm_panel_hide( panel[SELECT_LOCK] );
+   }
+}
 
 static void
 create_plugin(GtkWidget *vbox, gint first_create)
@@ -1427,7 +1471,10 @@ create_plugin(GtkWidget *vbox, gint first_create)
     
 	if (first_create)
 	{
-	  panel = gkrellm_panel_new0();
+          for ( i=0; i < 3; i++ )
+          {
+	    panel[i] = gkrellm_panel_new0();
+          }
 
 	  for ( i=0; i < MAX_PANELS; i++ )
 	  {
@@ -1436,8 +1483,11 @@ create_plugin(GtkWidget *vbox, gint first_create)
 	}  
 	else 
 	{ 
-	  gkrellm_destroy_krell_list(panel);
-	  gkrellm_destroy_decal_list(panel); 
+          for ( i=0; i < 3; i++ )
+          {
+	     gkrellm_destroy_krell_list(panel[i]);
+	     gkrellm_destroy_decal_list(panel[i]); 
+          }
 	}
 
 	for ( i=0; i < MAX_PANELS; i++ )
@@ -1456,35 +1506,53 @@ create_plugin(GtkWidget *vbox, gint first_create)
 	*/
 	ts = gkrellm_meter_textstyle(style_id);
 	ts_alt = gkrellm_meter_alt_textstyle(style_id);
-	panel->textstyle = ts;		/* would be used for a panel label */
+        for ( i=0; i < 3; i++ )
+        {
+	  panel[i]->textstyle = ts;/* would be used for a panel label */
+        }
 
        /* Create a text decal that will be converted to a button.  
 	* The "Lock" string is not an initialization string it is just a
 	* sizing string.
         */
-        decal_lock = gkrellm_create_decal_text(panel, "Lock", ts_alt, style, 2, 2, 0);
+        decal_lock[0] = gkrellm_create_decal_text(panel[0], "Lock", ts_alt, style, 2, 2, 0);
 
-	x = decal_lock->x + decal_lock->w + 6;
-	y = decal_lock->y + decal_lock->h + 2;
-        decal_shoot = gkrellm_create_decal_text(panel, "Shoot", ts_alt, style, x, 2, 0);
+	x = decal_lock[0]->x + decal_lock[0]->w + 6;
+	y = decal_lock[0]->y + decal_lock[0]->h + 2;
+        decal_shoot[0] = gkrellm_create_decal_text(panel[0], "Shoot", ts_alt, style, x, 2, 0);
+
+        decal_lock[1] = gkrellm_create_decal_text(panel[1], "L o c k", ts, style, 7, 2, 0);
+
+        decal_shoot[1] = gkrellm_create_decal_text(panel[2], "S h o o t", ts, style, 2, 2, 0);
 
 
 	/* Configure the panel to hold the above created decals, add in a little
 	|  bottom margin for looks, and create the panel.
 	*/
-        gkrellm_panel_configure(panel, NULL, style);
-        gkrellm_panel_create(vbox, mon, panel );
+        for ( i=0; i < 3; i++ )
+        {
+           gkrellm_panel_configure(panel[i], NULL, style);
+           gkrellm_panel_create(vbox, mon, panel[i] );
+        }
 
         /* After the panel is created, the decals can be converted into buttons.
         |  First draw the initial text into the text decal button and then
         |  put the text decal into a meter button.  
         */
-        gkrellm_draw_decal_text(panel, decal_lock, "Lock", 0 ); 
-        gkrellm_put_decal_in_meter_button(panel, decal_lock, cb_button,
+        gkrellm_draw_decal_text(panel[0], decal_lock[0], "Lock", 0 ); 
+        gkrellm_put_decal_in_meter_button(panel[0], decal_lock[0], cb_button,
                                 GINT_TO_POINTER(0), NULL);
 
-        gkrellm_draw_decal_text(panel, decal_shoot, "Shoot", 0 ); 
-        gkrellm_put_decal_in_meter_button(panel, decal_shoot, cb_button,
+        gkrellm_draw_decal_text(panel[0], decal_shoot[0], "Shoot", 0 ); 
+        gkrellm_put_decal_in_meter_button(panel[0], decal_shoot[0], cb_button,
+                                GINT_TO_POINTER(1), NULL);
+
+        gkrellm_draw_decal_text(panel[1], decal_lock[1], "L o c k", 0 ); 
+        gkrellm_put_decal_in_meter_button(panel[1], decal_lock[1], cb_button,
+                                GINT_TO_POINTER(0), NULL);
+
+        gkrellm_draw_decal_text(panel[2], decal_shoot[1], "S h o o t", 0 ); 
+        gkrellm_put_decal_in_meter_button(panel[2], decal_shoot[1], cb_button,
                                 GINT_TO_POINTER(1), NULL);
 
 	/* Note: all of the above gkrellm_draw_decal_XXX() calls will not
@@ -1493,13 +1561,17 @@ create_plugin(GtkWidget *vbox, gint first_create)
 	|  update_plugin().
 	*/
 
-        gkrellm_draw_panel_layers(panel);
+        for ( i=0; i < 3; i++ )
+        {
+          gkrellm_draw_panel_layers(panel[i]);
+        }
+        show_lock_shoot_select();
 
         if (shoot_tips == NULL)
         {
           shoot_tips = gtk_tooltips_new();
           shoot_tips_text = g_strdup("Click <Shoot> to grab window or screen\nClick <Lock> to lock the screen");
-          gtk_tooltips_set_tip(shoot_tips, panel->drawing_area,
+          gtk_tooltips_set_tip(shoot_tips, panel[0]->drawing_area,
                                shoot_tips_text, NULL);
           gtk_tooltips_set_delay(shoot_tips, 1000);
         }
@@ -1517,12 +1589,15 @@ create_plugin(GtkWidget *vbox, gint first_create)
 
 	if (first_create)
 	{	
-	    gtk_signal_connect(GTK_OBJECT (panel->drawing_area), "expose_event",
-    	        (GtkSignalFunc) expose_event, NULL);
+        for ( i = 0; i < 3; i++ )
+        {
+	    gtk_signal_connect(GTK_OBJECT (panel[i]->drawing_area), "expose_event",
+    	         (GtkSignalFunc) expose_event, GINT_TO_POINTER(i));
 
-	    gtk_signal_connect(GTK_OBJECT (panel->drawing_area), 
-		         "button_press_event",
-    	        (GtkSignalFunc) panel_press, NULL);
+	    gtk_signal_connect(GTK_OBJECT (panel[i]->drawing_area), 
+	 	         "button_press_event",
+    	         (GtkSignalFunc) panel_press, NULL);
+        }
 
         for ( i = 0; i < MAX_PANELS; i++ )
 		{
@@ -1566,6 +1641,10 @@ save_shoot_config (FILE *f)
     fprintf(f, "%s image_format %s\n", PLUGIN_CONFIG_KEYWORD, image_format );
     fprintf(f, "%s with_frame %d\n", PLUGIN_CONFIG_KEYWORD,
             with_frame);
+    fprintf(f, "%s grayscale %d\n", PLUGIN_CONFIG_KEYWORD, grayscale );
+    fprintf(f, "%s save_dir %s\n", PLUGIN_CONFIG_KEYWORD, save_dir);
+    fprintf(f, "%s lock_shoot_select %d\n", PLUGIN_CONFIG_KEYWORD,
+            lock_shoot_select);
 
 	for ( i=0; i < MAX_PANELS; i++ )
 	{
@@ -1640,6 +1719,14 @@ load_shoot_config (gchar *arg)
 		}  
         if (strcmp(config, "with_frame") == 0)
             sscanf(item, "%d\n", &(with_frame));
+ 	if (strcmp(config, "grayscale") == 0)
+ 	    sscanf(item, "%d\n", &(grayscale));
+        if (strcmp(config, "save_dir") == 0)
+	    {	
+              strcpy(save_dir, item);
+	    }    
+        if (strcmp(config, "lock_shoot_select") == 0)
+            sscanf(item, "%d\n", &(lock_shoot_select));
     }
 
 }
@@ -1658,6 +1745,8 @@ apply_shoot_config (void)
 
     with_frame = GTK_TOGGLE_BUTTON(frame_option)->active;
 
+    grayscale = GTK_TOGGLE_BUTTON(grayscale_option)->active;
+
     window_or_full = GTK_TOGGLE_BUTTON(window_option)->active;
 
     view_image = GTK_TOGGLE_BUTTON(view_image_option)->active;
@@ -1693,6 +1782,15 @@ apply_shoot_config (void)
     wait_seconds = gtk_spin_button_get_value_as_int(
             GTK_SPIN_BUTTON(wait_seconds_option));
 
+    c_text = gtk_entry_get_text(GTK_ENTRY(save_dir_option));
+
+    if ( g_file_test(c_text, G_FILE_TEST_IS_DIR ) )
+    {
+      if (strcmp(save_dir, c_text)) {
+  	  strcpy( save_dir, c_text );
+      }
+    }
+
 }
 static GtkWidget *create_anim_config_tab( gint i )
 {
@@ -1810,14 +1908,25 @@ static void num_panel_changed()
   change_num_panels ();
 }
 
+static void
+cb_lock_shoot_select(GtkWidget *button, gpointer data)
+{
+        gint    i = GPOINTER_TO_INT(data);
+
+        if (GTK_TOGGLE_BUTTON(button)->active)
+                lock_shoot_select = i;
+
+        show_lock_shoot_select();
+}
+
 static void
 create_shoot_tab (GtkWidget *tab)
 {
-    GtkWidget *frame, *ybox, *hbox, *vbox, *separator,
+    GtkWidget *frame, *ybox, *hbox, *vbox, *separator, 
               *label, *text, *info_window, *about_label;
     GtkAdjustment *wait_adjust, *num_panel_adjust;
 
-    GtkWidget *tablabel, *anim_config;
+    GtkWidget *tablabel, *anim_config, *button;
 	gchar *anim_tab_name;
 
     gint i;
@@ -1827,10 +1936,12 @@ create_shoot_tab (GtkWidget *tab)
     static gchar *help_text[] =
     {
       "<b>" CONFIG_NAME " " GKRELLSHOOT_VER "\n\n" ,
-      "Grabs screen/window when Shoot is clicked (uses ImageMagick) \n" ,
+      "Grabs screen/window when Shoot is clicked (uses ImageMagick 5.5.7 or greater) \n" ,
       "Locks the screen when Lock is clicked. And optionally will display " ,
       "Animation. \n\n" ,
       "<b> Options \n\n" ,
+      "Save Location - valid path to save images when shoot is clicked.\n",
+      "Lock/Shoot Select - choose the button to be displayed .\n",
       "Image Format - any valid image format supported by ImageMagick.\n",
       "Maximum of 3 animation panels are possible.Each can have its\n",
       "own animation. To Disable animation choose 0.\n\n",
@@ -1844,8 +1955,10 @@ create_shoot_tab (GtkWidget *tab)
       "appears on the screen using which one can select a window\n",
       "by clicking on a window or hold down mouse and choose an area \n",
       "to be grabbed, otherwise whole screen is grabbed. If with window \n",
-	  "frame is selected, windows are grabbed with their frame. Grabbed image is \n",
-      "saved on to current users home directory with the following name : \n",
+      "frame is selected, windows are grabbed with their frame. \n",
+      "If grayscale is selected you get grayscaled image. \n",
+      "Grabbed image is saved on to the directory specified in \n",
+      "Save Location with the following name : \n",
       "gkrellShoot_MM-DD-YY_HHMMSS.imageFormat(jpg,gif etc). \n\n",
       "If View Image is selected Grabbed Image is passed onto given ",
       "Image Viewer. "
@@ -1876,9 +1989,46 @@ create_shoot_tab (GtkWidget *tab)
 
     gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
+    /* SaveDir */
+    hbox = gtk_hbox_new(FALSE, 0);
+    label = gtk_label_new("Save Location " );
+    save_dir_option = gtk_entry_new_with_max_length(512);
+    gtk_entry_set_text(GTK_ENTRY(save_dir_option), save_dir);
+    gtk_entry_set_editable(GTK_ENTRY(save_dir_option), TRUE);
+
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), save_dir_option, TRUE, TRUE, 0);
+
+    gtk_container_add(GTK_CONTAINER(vbox), hbox);
+
     separator = gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 4);
 
+    /* Lock_Shoot Select */
+
+     ybox = gkrellm_gtk_framed_vbox(vbox, _("Lock/Shoot Select"),
+                        4, FALSE, 0, 2);
+        
+     hbox = gtk_hbox_new(FALSE, 0);
+     gtk_box_pack_start(GTK_BOX(ybox), hbox, FALSE, FALSE, 0);
+
+     lock_shoot_option[0] = gtk_radio_button_new_with_label(NULL, _("Both"));
+     gtk_box_pack_start(GTK_BOX(hbox), lock_shoot_option[0], TRUE, TRUE, 0);
+     lock_shoot_option[1] = gtk_radio_button_new_with_label_from_widget(
+                                        GTK_RADIO_BUTTON(lock_shoot_option[0]), _("Lock"));
+     gtk_box_pack_start(GTK_BOX(hbox), lock_shoot_option[1], TRUE, TRUE, 0);
+     lock_shoot_option[2] = gtk_radio_button_new_with_label_from_widget(
+                                        GTK_RADIO_BUTTON(lock_shoot_option[1]), _("Shoot"));
+     gtk_box_pack_start(GTK_BOX(hbox), lock_shoot_option[2], TRUE, TRUE, 0);
+
+     button = lock_shoot_option[lock_shoot_select];
+     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);
+        
+     for (i = 0; i < 3; ++i) {
+                g_signal_connect(G_OBJECT(lock_shoot_option[i]), "toggled",
+                                        G_CALLBACK(cb_lock_shoot_select), GINT_TO_POINTER(i));
+     }
+
     /* Number of Animation */
     hbox = gtk_hbox_new(FALSE, 0);
 
@@ -1915,6 +2065,11 @@ create_shoot_tab (GtkWidget *tab)
             with_frame );
     gtk_container_add(GTK_CONTAINER(vbox), frame_option);
 
+    grayscale_option = gtk_check_button_new_with_label( "grayscale" );
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(grayscale_option),
+            grayscale );
+    gtk_container_add(GTK_CONTAINER(vbox), grayscale_option);
+
     view_image_option = gtk_check_button_new_with_label( "View Image after click" );
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(view_image_option),
             view_image );
@@ -1932,6 +2087,8 @@ create_shoot_tab (GtkWidget *tab)
     gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
     ybox = gtk_hbox_new(FALSE, 0);
+    label = gtk_label_new("Seconds to wait before Screen Shot");
+    gtk_box_pack_start(GTK_BOX(ybox), label, FALSE, FALSE, 0);
     wait_adjust = (GtkAdjustment *) gtk_adjustment_new((gfloat)
             wait_seconds, 0, 180.0, 1.0, 5.0, 0.0);
     wait_seconds_option = gtk_spin_button_new(wait_adjust, 1.0, 1);
@@ -1940,10 +2097,8 @@ create_shoot_tab (GtkWidget *tab)
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(wait_seconds_option),
             wait_seconds);
     gtk_box_pack_start(GTK_BOX(ybox), wait_seconds_option, FALSE, FALSE, 0);
-    label = gtk_label_new(" seconds to wait before Screen Shot");
-    gtk_box_pack_start(GTK_BOX(ybox), label, FALSE, FALSE, 0);
 
-    label = gtk_label_new("Image Format" );
+    label = gtk_label_new(" Image Format" );
     image_format_option = gtk_entry_new_with_max_length(8);
     gtk_entry_set_text(GTK_ENTRY(image_format_option), image_format);
     gtk_entry_set_editable(GTK_ENTRY(image_format_option), TRUE);
@@ -1994,7 +2149,7 @@ create_shoot_tab (GtkWidget *tab)
         "GKrellShoot %s\n" \
         "GKrellM Shoot Plugin\n" \
         "\n" \
-        "Copyright (C) 2002 M.R.Muthu Kumar\n" \
+        "Copyright (C) 2004 M.R.Muthu Kumar\n" \
         "m_muthukumar@users.sourceforge.net\n" \
         "\n" \
         "Released under the GNU Public License\n" \
@@ -2057,7 +2212,10 @@ read_default(void)
    sprintf( view_cmd,"%s",  DEFAULT_VIEW );
    sprintf( image_format,"%s",  DEFAULT_IMAGE );
 
-   strcpy( filename, DEFAULT_OUTFILE );
+
+   sprintf( save_dir,"%s", gkrellm_homedir() );
+
+   sprintf( filename, "%s/%s", save_dir, DEFAULT_OUTFILE );
 
    gk_ticks = gkrellm_ticks();
 }	
